<?php

class SiteController extends Controller
{
	/**
	 * Declares class-based actions.
	 */
	public function actions()
	{
		return array(
			// captcha action renders the CAPTCHA image displayed on the contact page
			'captcha'=>array(
				'class'=>'CCaptchaAction',
				'backColor'=>0xFFFFFF,
			),
			// page action renders "static" pages stored under 'protected/views/site/pages'
			// They can be accessed via: index.php?r=site/page&view=FileName
			'page'=>array(
				'class'=>'CViewAction',
			),
		);
	}

	/**
	 * This is the default 'index' action that is invoked
	 * when an action is not explicitly requested by users.
	 */
	public function actionIndex()
	{
		// renders the view file 'protected/views/site/index.php'
		// using the default layout 'protected/views/layouts/main.php'
		$this->render('index');
	}

	/**
	 * This is the action to handle external exceptions.
	 */
	public function actionError()
	{
	    if($error=Yii::app()->errorHandler->error)
	    {
	    	if(Yii::app()->request->isAjaxRequest)
	    		echo $error['message'];
	    	else
	        	$this->render('error', $error);
	    }
	}

	/**
	 * Displays the contact page
	 */
	public function actionContact()
	{
		$model=new ContactForm;
		if(isset($_POST['ContactForm']))
		{
			$model->attributes=$_POST['ContactForm'];
			if($model->validate())
			{
				$headers="From: {$model->email}\r\nReply-To: {$model->email}";
				mail(Yii::app()->params['adminEmail'],$model->subject,$model->body,$headers);
				Yii::app()->user->setFlash('contact','Thank you for contacting us. We will respond to you as soon as possible.');
				$this->refresh();
			}
		}
		$this->render('contact',array('model'=>$model));
	}

	/**
	 * Displays the login page
	 */
	public function actionLogin()
	{
		$model=new LoginForm;

		// if it is ajax validation request
		if(isset($_POST['ajax']) && $_POST['ajax']==='login-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}

		// collect user input data
		if(isset($_POST['LoginForm']))
		{
			$model->attributes=$_POST['LoginForm'];
			// validate user input and redirect to the previous page if valid
			if($model->validate() && $model->login())
				$this->redirect(Yii::app()->user->returnUrl);
		}
		// display the login form
		$this->render('login',array('model'=>$model));
	}

	/**
	 * Logs out the current user and redirect to homepage.
	 */
	public function actionLogout()
	{
		Yii::app()->user->logout();
		$this->redirect(Yii::app()->homeUrl);
	}
	
	public function actionJSONSpeedTest()
	{
		$cycles = 10;
		$time_taken_model = 0;
		$time_taken_command = 0;
		$time_taken_pdo = 0;
		$time_taken_command_query_builder = 0;
		for($i = 0; $i < $cycles; $i++)
		{
			$time_taken_command += $this->testCommand();
			$time_taken_pdo += $this->testPDO();
			$time_taken_command_query_builder += $this->testCommandQueryBuilder();
			$time_taken_model += $this->testModel();			
		}
		$time_taken_model /= $cycles;
		$time_taken_command /= $cycles;		
		$time_taken_pdo /= $cycles;	
		$time_taken_command_query_builder /= $cycles;			
		
		$this->render('jsonspeedtest', array(
			"time_taken_pdo" => $time_taken_pdo,				
			"time_taken_command" => $time_taken_command,			
			"time_taken_model" => $time_taken_model,
			"time_taken_command_query_builder" => $time_taken_command_query_builder,
			"cycles" => $cycles,
		));
	}
	
	private function testPDO(){
		$start_time = microtime(true);
		
		$connection = Yii::app()->components['db']->connectionString;
		$username = Yii::app()->components['db']->username;
		$password = Yii::app()->components['db']->password;
		$db = new PDO($connection, $username, $password);
		
		$table = 'film_text';
		$columns = array('film_id', 'title', 'description');
		$sql =  "SELECT " . implode(",", $columns) . " FROM " . $table;		
		
		$query = $db->prepare($sql);
		$query->execute();
		$row_results = $query->fetchAll(PDO::FETCH_NUM);
		$json = $this->resultsToJSON($table, $columns, $row_results);
		
		$end_time = microtime(true);
		$time_taken = $end_time - $start_time;
		return $time_taken;
	}
	
	private function testCommand(){
		
		$start_time = microtime(true);
		
		$table = 'film_text';
		$columns = array('film_id', 'title', 'description');		
		
		$connection = Yii::app()->db;
		$sql =  "SELECT " . implode(",", $columns) . " FROM " . $table;
		$command = $connection->createCommand($sql);
		$row_results = $command->queryAll(false);
		
		$json = $this->resultsToJSON($table, $columns, $row_results);
		
		$end_time = microtime(true);
		$time_taken = $end_time - $start_time;
		return $time_taken;
	}
	
	private function testCommandQueryBuilder(){
		
		$start_time = microtime(true);
		
		$table = 'film_text';
		$columns = array('film_id', 'title', 'description');		
		
		$connection = Yii::app()->db;
		$cmd =  "SELECT " . implode(",", $columns) . " FROM " . $table;
		$row_results = Yii::app()->db->createCommand()
							->select(implode(",", $columns))
							->from($table)
							->queryAll();
		
		$json = $this->resultsToJSON($table, $columns, $row_results);
		
		$end_time = microtime(true);
		$time_taken = $end_time - $start_time;
		return $time_taken;
	}	
	
	private function resultsToJSON($table, $columns, $results){
		
		$json = '{"' . $table . '" : {';
		
		$json .= '"columns" : ["' . implode('","', $columns) . '"],';
		
		$json .= '"rows" : [';		
		foreach($results as $row){
			$json_row = '["';
			$json_row .= implode('","', $row);
			$json_row .= '"],';				
			$json .= $json_row;
		}
		$json = substr($json, 0, -1);	// Remove last comma
		$json .= ']';
		$json .= '}}';
		return $json;
	}
	
	private function testModel(){
		
		$start_time = microtime(true);
		
		$models = FilmText::model()->findAll();
		$json = $this->modelsToJSON($models);
		
		$end_time = microtime(true);
		$time_taken = $end_time - $start_time;
		return $time_taken;
	}
	
	private function modelsToJSON($models)
	{
		if(empty($models))
			throw new Exception("No rows in the model.");	
		
		$json = '{"' . $models[0]->tableAlias . '" : {';
		
		$json .= '"columns" : [';
		$meta_data = $models[0]->getMetaData()->columns;
		foreach($meta_data as $column){
			$json .= '"' . $column->name . '",';
		}
		$json = substr($json, 0, -1);	// Remove last comma
		$json .= '],';
		
		$json .= '"rows" : [';		
		foreach($models as $model){
			$json_row = "[";		
			$attributes = $model->getAttributes(null);		
			foreach($attributes as $attribute){
				$json_row .= '"' . $attribute . '",';
			}
			$json_row = substr($json_row, 0, -1);	// Remove last comma
			$json_row .= "],";				
			$json .= $json_row;
		}
		$json = substr($json, 0, -1);	// Remove last comma
		$json .= ']';
		$json .= '}}';
		return $json;
	}
	
	
}